@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1995-2021, University of Amsterdam
+    Copyright (c)  1995-2024, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
                               SWI-Prolog Solutions b.v.
@@ -62,7 +62,7 @@ also used by the commandline sequence below.
 
 :- multifile error:has_type/2.
 error:has_type(qsave_foreign_option, Term) :-
-    is_of_type(oneof([save, no_save]), Term),
+    is_of_type(oneof([save, no_save, copy]), Term),
     !.
 error:has_type(qsave_foreign_option, arch(Archs)) :-
     is_of_type(list(atom), Archs),
@@ -151,7 +151,7 @@ qsave_program(FileBase, Options0) :-
                                      % running on this state
           setup_call_catcher_cleanup(
               open(File, write, StateOut, [type(binary)]),
-              write_state(StateOut, SaveClass, Options),
+              write_state(StateOut, SaveClass, File, Options),
               Reason,
               finalize_state(Reason, StateOut, File))
         ),
@@ -159,20 +159,20 @@ qsave_program(FileBase, Options0) :-
     cleanup,
     !.
 
-write_state(StateOut, SaveClass, Options) :-
+write_state(StateOut, SaveClass, ExeFile, Options) :-
     make_header(StateOut, SaveClass, Options),
     setup_call_cleanup(
         zip_open_stream(StateOut, RC, []),
-        write_zip_state(RC, SaveClass, Options),
+        write_zip_state(RC, SaveClass, ExeFile, Options),
         zip_close(RC, [comment('SWI-Prolog saved state')])),
     flush_output(StateOut).
 
-write_zip_state(RC, SaveClass, Options) :-
+write_zip_state(RC, SaveClass, ExeFile, Options) :-
     save_options(RC, SaveClass, Options),
     save_resources(RC, SaveClass),
     lock_files(SaveClass),
     save_program(RC, SaveClass, Options),
-    save_foreign_libraries(RC, Options).
+    save_foreign_libraries(RC, ExeFile, Options).
 
 finalize_state(exit, StateOut, File) :-
     close(StateOut),
@@ -220,29 +220,17 @@ qsave_init_file_option(_, Options, Options).
 %!  make_header(+Out:stream, +SaveClass, +Options) is det.
 
 make_header(Out, _, Options) :-
-    option(emulator(OptVal), Options),
+    stand_alone(Options),
     !,
-    absolute_file_name(OptVal, [access(read)], Emulator),
+    emulator(Emulator, Options),
     setup_call_cleanup(
         open(Emulator, read, In, [type(binary)]),
         copy_stream_data(In, Out),
         close(In)).
-make_header(Out, _, Options) :-
-    (   current_prolog_flag(windows, true)
-    ->  DefStandAlone = true
-    ;   DefStandAlone = false
-    ),
-    option(stand_alone(true), Options, DefStandAlone),
-    !,
-    current_prolog_flag(executable, Executable),
-    setup_call_cleanup(
-        open(Executable, read, In, [type(binary)]),
-        copy_stream_data(In, Out),
-        close(In)).
-make_header(Out, SaveClass, _Options) :-
+make_header(Out, SaveClass, Options) :-
     current_prolog_flag(unix, true),
     !,
-    current_prolog_flag(executable, Executable),
+    emulator(Emulator, Options),
     current_prolog_flag(posix_shell, Shell),
     format(Out, '#!~w~n', [Shell]),
     format(Out, '# SWI-Prolog saved state~n', []),
@@ -250,9 +238,23 @@ make_header(Out, SaveClass, _Options) :-
     ->  ArgSep = ' -- '
     ;   ArgSep = ' '
     ),
-    format(Out, 'exec ${SWIPL-~w} -x "$0"~w"$@"~n~n', [Executable, ArgSep]).
+    format(Out, 'exec ${SWIPL:-~w} -x "$0"~w"$@"~n~n', [Emulator, ArgSep]).
 make_header(_, _, _).
 
+stand_alone(Options) :-
+    (   current_prolog_flag(windows, true)
+    ->  DefStandAlone = true
+    ;   DefStandAlone = false
+    ),
+    option(stand_alone(true), Options, DefStandAlone).
+
+emulator(Emulator, Options) :-
+    (   option(emulator(OptVal), Options)
+    ->  absolute_file_name(OptVal, [access(read)], Emulator)
+    ;   current_prolog_flag(executable, Emulator)
+    ).
+
+
 
                  /*******************************
                  *           OPTIONS            *
@@ -670,7 +672,8 @@ save_predicate(P, _SaveClass) :-
     P = (M:H),
     functor(H, Name, Arity),
     feedback('~npre-defining foreign ~w/~d ', [Name, Arity]),
-    '$add_directive_wic'('$predefine_foreign'(M:Name/Arity)).
+    '$add_directive_wic'('$predefine_foreign'(M:Name/Arity)),
+    save_attributes(P).
 save_predicate(P, SaveClass) :-
     P = (M:H),
     functor(H, F, A),
@@ -716,6 +719,7 @@ pred_attrib(Attrib, Head,
     functor(Term, Name, Arity).
 
 attrib_name(dynamic,                dynamic,                true).
+attrib_name(incremental,            incremental,            true).
 attrib_name(volatile,               volatile,               true).
 attrib_name(thread_local,           thread_local,           true).
 attrib_name(multifile,              multifile,              true).
@@ -938,24 +942,28 @@ qualify_head(T, user:T).
                  *       FOREIGN LIBRARIES      *
                  *******************************/
 
-%!  save_foreign_libraries(+Archive, +Options) is det.
+%!  save_foreign_libraries(+Archive, +ExeFile, +Options) is det.
 %
 %   Save current foreign libraries into the archive.
 
-save_foreign_libraries(RC, Options) :-
+save_foreign_libraries(RC, _, Options) :-
     option(foreign(save), Options),
     !,
     current_prolog_flag(arch, HostArch),
     feedback('~nHOST(~w) FOREIGN LIBRARIES~n', [HostArch]),
     save_foreign_libraries1(HostArch, RC, Options).
-save_foreign_libraries(RC, Options) :-
+save_foreign_libraries(RC, _, Options) :-
     option(foreign(arch(Archs)), Options),
     !,
     forall(member(Arch, Archs),
            ( feedback('~n~w FOREIGN LIBRARIES~n', [Arch]),
              save_foreign_libraries1(Arch, RC, Options)
            )).
-save_foreign_libraries(_, _).
+save_foreign_libraries(_RC, ExeFile, Options) :-
+    option(foreign(copy), Options),
+    !,
+    copy_foreign_libraries(ExeFile, Options).
+save_foreign_libraries(_, _, _).
 
 save_foreign_libraries1(Arch, RC, _Options) :-
     forall(current_foreign_library(FileSpec, _Predicates),
@@ -964,6 +972,35 @@ save_foreign_libraries1(Arch, RC, _Options) :-
              zipper_append_file(RC, Name, File, [time(Time)])
            )).
 
+%!  copy_foreign_libraries(+Exe, +Options) is det.
+%
+%   Copy all required foreign libraries   to  an installation directory.
+%   This is currently only implemented  for   Windows,  copying all .dll
+%   files to the directory where the executable is created.
+
+:- if(current_prolog_flag(windows, true)).
+copy_foreign_libraries(ExeFile, _Options) :-
+    !,
+    file_directory_name(ExeFile, Dir),
+    win_process_modules(Modules),
+    include(prolog_dll, Modules, PrologDLLs),
+    maplist(copy_dll(Dir), PrologDLLs).
+:- endif.
+copy_foreign_libraries(_ExeFile, _Options) :-
+    print_message(warning, qsave(copy_foreign_libraries)).
+
+prolog_dll(DLL) :-
+    file_base_name(DLL, File),
+    absolute_file_name(foreign(File), Abs,
+                       [ solutions(all) ]),
+	same_file(DLL, Abs),
+    !.
+
+copy_dll(Dest, DLL) :-
+    print_message(informational, copy_foreign_library(DLL, Dest)),
+    copy_file(DLL, Dest).
+
+
 %!  find_foreign_library(+Architecture, +FileSpec, -EntryName, -File, -Time)
 %!								is det.
 %
@@ -1314,7 +1351,7 @@ qsave_option(Name, Name, ValueStrings, Value) :-
 qsave_option(Name, Name, _Chars, _Value) :-
     existence_error(save_option, Name).
 
-convert_option_value(integer, String, Value) :-
+convert_option_value(integer, String, Value) =>
     (   number_string(Value, String)
     ->  true
     ;   sub_string(String, 0, _, 1, SubString),
@@ -1325,18 +1362,21 @@ convert_option_value(integer, String, Value) :-
     ->  Value is Number * Multiplier
     ;   domain_error(integer, String)
     ).
-convert_option_value(callable, String, Value) :-
+convert_option_value(callable, String, Value) =>
     term_string(Value, String).
-convert_option_value(atom, String, Value) :-
+convert_option_value(atom, String, Value) =>
     atom_string(Value, String).
-convert_option_value(boolean, String, Value) :-
+convert_option_value(boolean, String, Value) =>
     atom_string(Value, String).
-convert_option_value(oneof(_), String, Value) :-
+convert_option_value(oneof(_), String, Value) =>
     atom_string(Value, String).
-convert_option_value(ground, String, Value) :-
+convert_option_value(ground, String, Value) =>
     atom_string(Value, String).
-convert_option_value(qsave_foreign_option, "save", save).
-convert_option_value(qsave_foreign_option, StrArchList, arch(ArchList)) :-
+convert_option_value(qsave_foreign_option, "save", Value) =>
+    Value = save.
+convert_option_value(qsave_foreign_option, "copy", Value) =>
+    Value = copy.
+convert_option_value(qsave_foreign_option, StrArchList, arch(ArchList)) =>
     split_string(StrArchList, ",", ", \t", StrArchList1),
     maplist(atom_string, ArchList, StrArchList1).
 
@@ -1357,3 +1397,5 @@ prolog:message(no_resource(Name, File)) -->
       [Name, File] ].
 prolog:message(qsave(nondet)) -->
     [ 'qsave_program/2 succeeded with a choice point'-[] ].
+prolog:message(copy_foreign_library(Lib,Dir)) -->
+    [ 'Copying ~w to ~w'-[Lib, Dir] ].
