@@ -59,8 +59,8 @@ mqi_start([unix_domain_socket(Socket), password('a password')])
 ~~~
 When using ["Embedded Mode"](#mqi-embedded-mode) they are passed using the same name but as normal command line arguments like this:
 ~~~
-swipl --quiet -g mqi_start -t halt -- --write_connection_values=true
---password="a password" --create_unix_domain_socket=true
+swipl mqi --write_connection_values=true
+          --password="a password" --create_unix_domain_socket=true
 ~~~
 
 Note the use of quotes around values that could confuse command line
@@ -167,10 +167,11 @@ Redirects STDOUT and STDERR to the file path specified.  Useful for debugging th
 %  have the broadest adoption possible.
 %
 %  Protocol Version History:
-%  0.0 ->   First published version. Had a protocol bug that required messages sent to MQI to
-%           count Unicode code points instead of bytes for the message header.
-%  1.0 ->   Breaking change: Fixed protocol bug so that it properly accepted byte count instead of Unicode code point
-%           count in the message header for messages sent to MQI.
+%  - 0.0 First published version. Had a protocol bug that required messages sent to MQI to
+%        count Unicode code points instead of bytes for the message header.
+%
+%  - 1.0 Breaking change: Fixed protocol bug so that it properly accepted byte count instead of Unicode code point
+%        count in the message header for messages sent to MQI.
 mqi_version(1, 0).
 
 
@@ -255,7 +256,7 @@ opt_help(write_output_to_file,      "Write stdout and stderr to file").
 %  To launch embedded mode:
 %
 %  ~~~
-%  swipl --quiet -g mqi_start -t halt -- --write_connection_values=true
+%  swipl mqi --write_connection_values=true
 %  ~~~
 %
 %  This will start SWI Prolog and   invoke the mqi_start/0 predicate and
@@ -269,14 +270,13 @@ opt_help(write_output_to_file,      "Write stdout and stderr to file").
 %  as spaces) should be passed with =|""|= like this:
 %
 %  ~~~
-%  swipl --quiet -g mqi_start -t halt -- --write_connection_values=true \
-%                                        --password="HGJ SOWLWW WNDSJD"
+%  swipl mqi --write_connection_values=true --password="HGJ SOWLWW"
 %  ~~~
 %
 %  For help on commandline options run
 %
 %  ~~~
-%  swipl -g mqi_start -- --help
+%  swipl mqi --help
 %  ~~~
 
 
@@ -323,19 +323,21 @@ quit(_) :-
 mqi_stop(Server_Thread_ID) :-
     % First shut down any matching servers to stop new connections
     forall(retract(mqi_thread(Server_Thread_ID, _, Socket)),
-        (
-            debug(mqi(protocol), "Found server: ~w", [Server_Thread_ID]),
-            catch(tcp_close_socket(Socket), Socket_Exception, true),
-            abortSilentExit(Server_Thread_ID, Server_Thread_Exception),
-            debug(mqi(protocol), "Stopped server thread: ~w, socket_close_exception(~w), stop_thread_exception(~w)", [Server_Thread_ID, Socket_Exception, Server_Thread_Exception])
-        )),
+           (   debug(mqi(protocol), "Found server: ~w", [Server_Thread_ID]),
+               catch(tcp_close_socket(Socket), _0Socket_Exception, true),
+               abortSilentExit(Server_Thread_ID, _0Server_Thread_Exception),
+               debug(mqi(protocol), "Stopped server thread: ~w, \c
+                                     socket_close_exception(~w), stop_thread_exception(~w)",
+                     [Server_Thread_ID, _0Socket_Exception, _0Server_Thread_Exception])
+           )),
     forall(retract(mqi_worker_threads(Server_Thread_ID, Communication_Thread_ID, Goal_Thread_ID)),
-        (
-            abortSilentExit(Communication_Thread_ID, CommunicationException),
-            debug(mqi(protocol), "Stopped server: ~w communication thread: ~w, exception(~w)", [Server_Thread_ID, Communication_Thread_ID, CommunicationException]),
-            abortSilentExit(Goal_Thread_ID, Goal_Exception),
-            debug(mqi(protocol), "Stopped server: ~w goal thread: ~w, exception(~w)", [Server_Thread_ID, Goal_Thread_ID, Goal_Exception])
-        )).
+           (   abortSilentExit(Communication_Thread_ID, _0CommunicationException),
+               debug(mqi(protocol), "Stopped server: ~w communication thread: ~w, exception(~w)",
+                     [Server_Thread_ID, Communication_Thread_ID, _0CommunicationException]),
+               abortSilentExit(Goal_Thread_ID, _0Goal_Exception),
+               debug(mqi(protocol), "Stopped server: ~w goal thread: ~w, exception(~w)",
+                     [Server_Thread_ID, Goal_Thread_ID, _0Goal_Exception])
+           )).
 
 
 start_server_thread(Run_Server_On_Thread, Server_Thread_ID, Server_Goal, Unix_Domain_Socket_Path, Unix_Domain_Socket_Path_And_File) :-
@@ -453,19 +455,27 @@ goal_thread(Respond_To_Thread_ID) :-
     debug(mqi(query), "Received Findall = ~w, Query_Timeout = ~w, binding list: ~w, unexpanded: ~w, goal: ~w", [Find_All, Query_Timeout, Binding_List, Unexpanded_Goal, Goal]),
     (   Find_All
     ->  One_Answer_Goal = findall(Binding_List, @(user:Goal, user), Answers)
-    ;   One_Answer_Goal = ( @(user:Goal, user),
-                            Answers = [Binding_List],
-                            send_next_result(Respond_To_Thread_ID, Answers, _, Find_All)
+    ;   One_Answer_Goal = ( findall(    One_Answer,
+                                        (   @(user:Goal, user),
+                                            One_Answer = [Binding_List],
+                                            send_next_result(Respond_To_Thread_ID, One_Answer, _, Find_All)
+                                        ),
+                                        Answers
+                            ),
+                            (   Answers == []
+                            ->  send_next_result(Respond_To_Thread_ID, [], _, Find_All)
+                            ;   true
+                            )
                           )
     ),
-    All_Answers_Goal = run_cancellable_goal(Self_ID, findall(Answers, One_Answer_Goal, [Find_All_Answers | _])),
+    Cancellable_Goal = run_cancellable_goal(Self_ID, One_Answer_Goal),
     (   Query_Timeout == -1
-    ->  catch(All_Answers_Goal, Top_Exception, true)
-    ;   catch(call_with_time_limit(Query_Timeout, All_Answers_Goal), Top_Exception, true)
+    ->  catch(Cancellable_Goal, Top_Exception, true)
+    ;   catch(call_with_time_limit(Query_Timeout, Cancellable_Goal), Top_Exception, true)
     ),
     (   var(Top_Exception)
     ->  (   Find_All
-        ->  send_next_result(Respond_To_Thread_ID, Find_All_Answers, _, Find_All)
+        ->  send_next_result(Respond_To_Thread_ID, Answers, _, Find_All)
         ;   send_next_result(Respond_To_Thread_ID, [], no_more_results, Find_All)
         )
     ;   send_next_result(Respond_To_Thread_ID, [], Top_Exception, true)
@@ -511,22 +521,22 @@ run_cancellable_goal(Mutex_ID, Goal) :-
 %   otherwise just exit the session
 communication_thread(Password, Socket, Encoding, Server_Thread_ID, Goal_Thread_ID, Query_Timeout, Exit_Main_On_Failure) :-
     thread_self(Self_ID),
-    (   (
-            catch(communication_thread_listen(Password, Socket, Encoding, Server_Thread_ID, Goal_Thread_ID, Query_Timeout), error(Serve_Exception1, Serve_Exception2), true),
-            debug(mqi(protocol), "Session finished. Communication thread exception: ~w", [error(Serve_Exception1, Serve_Exception2)]),
-            abortSilentExit(Goal_Thread_ID, _),
-            retractall(mqi_worker_threads(Server_Thread_ID, Self_ID, Goal_Thread_ID))
-        )
+    (   catch(communication_thread_listen(Password, Socket, Encoding, Server_Thread_ID, Goal_Thread_ID, Query_Timeout),
+              error(Serve_Exception1, _0Serve_Exception2),
+              true),
+        debug(mqi(protocol), "Session finished. Communication thread exception: ~w",
+              [error(Serve_Exception1, _0Serve_Exception2)]),
+        abortSilentExit(Goal_Thread_ID, _),
+        retractall(mqi_worker_threads(Server_Thread_ID, Self_ID, Goal_Thread_ID))
     ->  Halt = (nonvar(Serve_Exception1), Exit_Main_On_Failure)
     ;   Halt = true
     ),
     (   Halt
-    ->  (   debug(mqi(protocol), "Ending session and halting Prolog server due to thread ~w: exception(~w)", [Self_ID, error(Serve_Exception1, Serve_Exception2)]),
-            quit(_)
-        )
-    ;   (   debug(mqi(protocol), "Ending session ~w", [Self_ID]),
-            catch(tcp_close_socket(Socket), error(_, _), true)
-        )
+    ->  debug(mqi(protocol), "Ending session and halting Prolog server due to thread ~w: exception(~w)",
+              [Self_ID, error(Serve_Exception1, _0Serve_Exception2)]),
+        quit(_)
+    ;   debug(mqi(protocol), "Ending session ~w", [Self_ID]),
+        catch(tcp_close_socket(Socket), error(_, _), true)
     ).
 
 
@@ -821,7 +831,8 @@ send_next_result(Respond_To_Thread_ID, Answer, Exception_In_Goal, Find_All) :-
                 Answer == []
             )
         ->  thread_send_message(Respond_To_Thread_ID, result(false, Find_All))
-        ;   thread_send_message(Respond_To_Thread_ID, result(true(Answer), Find_All))
+        ;   handle_constraints(Answer, Final_Answer),
+            thread_send_message(Respond_To_Thread_ID, result(true(Final_Answer), Find_All))
         )
     ;   (   debug(mqi(query), "Sending result of goal to communication thread, Exception: ~w", [Exception_In_Goal]),
             thread_send_message(Respond_To_Thread_ID, result(error(Exception_In_Goal), Find_All))
@@ -829,10 +840,24 @@ send_next_result(Respond_To_Thread_ID, Answer, Exception_In_Goal, Find_All) :-
     ).
 
 
+handle_constraints(Answer, Final_Answer) :-
+    (   term_attvars(Answer, [])
+    ->  Final_Answer = Answer
+    ;   findall(    Single_Answer_With_Attributes,
+                    (   member(Single_Answer, Answer),
+                        copy_term(Single_Answer, Single_Answer_Copy, Attributes),
+                        append(['$residuals' = Attributes], Single_Answer_Copy, Single_Answer_With_Attributes)
+                    ),
+                    Final_Answer
+        ),
+        debug(mqi(query), "Constraints detected, converted: ~w to ~w", [Answer, Final_Answer])
+    ).
+
+
 % Gets the next result from the goal thread in the communication thread queue,
 % and retracts query_in_progress/1 when the last result has been sent.
 % Find_All == true only returns one message, so delete query_in_progress
-% No matter what it is
+% no matter what it is
 % \+ Find_All: There may be more than one result. The first one we hit with any exception
 % (note that no_more_results is also returned as an exception) means we are done
 get_next_result(Goal_Thread_ID, Stream, Options, Answers) :-
@@ -862,9 +887,11 @@ get_next_result(Goal_Thread_ID, Stream, Options, Answers) :-
 reply_with_result(_, Stream, error(Error)) :-
     !,
     reply_error(Stream, Error).
+
+% Gracefully handle exceptions that can occur during conversion to JSON
 reply_with_result(_, Stream, Result) :-
     !,
-    reply(Stream, Result).
+    catch(reply(Stream, Result), error(Exception, _), reply_with_result(_, Stream, error(Exception))).
 
 
 % Reply with a normal term
@@ -972,15 +999,6 @@ repeat_until_false(_).
 abortSilentExit(Thread_ID, Exception) :-
     catch(thread_signal(Thread_ID, abort), error(Exception, _), true),
     debug(mqi(protocol), "Attempting to abort thread: ~w. thread_signal_exception: ~w", [Thread_ID, Exception]).
-% Workaround SWI Prolog bug: https://github.com/SWI-Prolog/swipl-devel/issues/852 by not joining
-% The workaround just stops joining the aborted thread, so an inert record will be left if thread_property/2 is called.
-%    ,
-%    (   once((var(Exception) ; catch(thread_property(Thread_ID, status(exception('$aborted'))), error(_, _), true)))
-%    ->  (   catch(call_with_time_limit(4, thread_join(Thread_ID)), error(JoinException1, JoinException2), true),
-%            debug(mqi(protocol), "thread_join attempted because thread: ~w exit was expected, exception: ~w", [Thread_ID, error(JoinException1, JoinException2)])
-%        )
-%    ;   true
-%    ).
 
 
 % Detach a thread that exits with true or false so that it doesn't leave around a record in thread_property/2 afterwards
@@ -1040,23 +1058,3 @@ unix_domain_socket_path(Created_Directory, File_Path) :-
                         set_prolog_flag(tmp_dir, Save_Tmp_Dir)
                       ),
     close(Stream).
-
-
-% Helper for installing the mqi.pl file to the right
-% library directory.
-% Call using swipl -s mqi.pl -g "mqi:install_to_library('mqi.pl')" -t halt
-install_to_library(File) :-
-    once(find_library(Path)),
-    copy_file(File, Path),
-    make.
-
-
-% Find the base library path, i.e. the one that ends in
-% "library/"
-find_library(Path) :-
-    file_alias_path(library, Path),
-    atomic_list_concat(Parts, '/', Path),
-    reverse(Parts, Parts_Reverse),
-    nth0(0, Parts_Reverse, ''),
-    nth0(1, Parts_Reverse, Library),
-    string_lower(Library, 'library').
