@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1999-2018, University of Amsterdam
+    Copyright (c)  1999-2025, University of Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -33,6 +34,7 @@
     POSSIBILITY OF SUCH DAMAGE.
 */
 
+#define _CRT_SECURE_NO_WARNINGS 1
 #include <config.h>
 
 #define O_ORDER				/* include order.c package */
@@ -46,6 +48,7 @@
 #include "table.h"
 #include "error.h"
 #include <stdlib.h>
+#include <stdbool.h>
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif
@@ -330,7 +333,7 @@ get_offset_ex(term_t t, table_offset_t *v)
 }
 
 
-static int
+static bool
 get_table_ex(term_t handle, Table *table)
 { int64_t l;
 
@@ -342,16 +345,16 @@ get_table_ex(term_t handle, Table *table)
       return TRUE;
     }
 
-    return existence_error(handle, "table");
+    return existence_error(handle, "table"),false;
   }
 
-  return type_error(handle, "table");
+  return type_error(handle, "table"),false;
 }
 
 
-static int
+static bool
 get_type(atom_t typename, int *type)
-{      if ( typename == ATOM_atom )
+{ if ( typename == ATOM_atom )
     *type = FIELD_ATOM;
   else if ( typename == ATOM_string )
     *type = FIELD_STRING;
@@ -364,9 +367,9 @@ get_type(atom_t typename, int *type)
   else if ( typename == ATOM_float )
     *type = FIELD_FLOAT;
   else
-    return FALSE;
+    return false;
 
-  return TRUE;
+  return true;
 }
 
 
@@ -1034,7 +1037,7 @@ find_start_of_record(Table t, table_offset_t start)
 { char *s;
   int er = t->record_sep;
 
-  if ( start < 0 || start > (table_offset_t)t->window_size )
+  if ( start < 0 || start >= (table_offset_t)t->window_size )
     return -1;
 
   if ( start == (table_offset_t)t->window_size && start > 0 )
