@@ -363,7 +363,7 @@ pasteTextItem(TextItem ti, Name buffer)
 static Browser Completer = NULL;
 
 Browser
-CompletionBrowser()
+CompletionBrowser(void)
 { if ( !Completer )
   { KeyBinding kb;
     Any client;
@@ -373,8 +373,6 @@ CompletionBrowser()
     client    = newObject(ClassObtain, Completer, NAME_client, EAV);
     quit      = newObject(ClassMessage, client, NAME_keyboardQuit, EAV);
 
-    protectObject(Completer);
-    protectObject(Completer->frame);
     attributeObject(Completer, NAME_client, NIL);
     attributeObject(Completer, NAME_prefix, NAME_);
     attributeObject(Completer, NAME_autoHide, ZERO);
@@ -394,6 +392,9 @@ CompletionBrowser()
     functionKeyBinding(kb, CtoName("\\C-g"), quit);
     functionKeyBinding(kb, CtoName("\\e"),   quit);
     functionKeyBinding(kb, CtoName("SPC"),   NAME_extendPrefix);
+
+    protectObject(Completer);
+    protectObject(Completer->frame);
   }
 
   return Completer;
@@ -409,25 +410,42 @@ completerShownDialogItem(Any di)
 }
 
 
-status
-quitCompleterDialogItem(Any di)
-{ if ( completerShownDialogItem(di) )
-  { PceWindow sw;
-    Browser c = CompletionBrowser();
+static void
+destroyCompleter(Browser c)
+{ Any di = getAttributeObject(c, NAME_client);
 
-    if ( (sw = getWindowGraphical(di)) )
+  if ( di )
+  { PceWindow sw = getWindowGraphical(di);
+
+    if ( sw )
     { grabPointerWindow(sw, OFF);
       focusWindow(sw, NIL, NIL, NIL, NIL);
     }
-
-    send(c, NAME_clear, EAV);
-    send(c, NAME_client, NIL, EAV);
-    send(c, NAME_show, OFF, EAV);
-    send(c, NAME_transientFor, NIL, EAV);
-    if ( text_item_combo_width(di) )
-      changedDialogItem(di);		/* indicator will change */
   }
 
+  send(c, NAME_clear, EAV);
+  send(c, NAME_client, NIL, EAV);
+  send(c, NAME_show, OFF, EAV);
+  send(c, NAME_transientFor, NIL, EAV);
+  if ( di && text_item_combo_width(di) )
+    changedDialogItem(di);		/* indicator will change */
+}
+
+bool
+destroyCompleterFrame(FrameObj fr)
+{ if ( Completer && getFrameWindow((PceWindow)Completer, OFF) == fr )
+  { destroyCompleter(Completer);
+    return true;
+  }
+
+  return false;
+}
+
+status
+quitCompleterDialogItem(Any di)
+{ if ( completerShownDialogItem(di) )
+    destroyCompleter(CompletionBrowser());
+
   succeed;
 }
 
@@ -977,7 +995,7 @@ eventTextItem(TextItem ti, EventObj ev)
 	  if ( autohide != ZERO )
 	  { StringObj ss = lb->search_string;
 	    if ( isNil(ss) || valInt(autohide) > valInt(getSizeCharArray(ss)) )
-	    quitCompleterDialogItem(ti);
+	      quitCompleterDialogItem(ti);
 	  }
 	}
 
