@@ -35,6 +35,7 @@
 
 #include <h/kernel.h>
 #include <h/graphics.h>
+#include <h/dialog.h>
 
 TileObj		getTileFrame(FrameObj);
 forwards int	get_position_from_center_frame(FrameObj, DisplayObj, Point, int *, int *);
@@ -93,6 +94,15 @@ initialiseFrame(FrameObj fr, Name label, Name kind,
   succeed;
 }
 
+static void
+destroyTransientFrame(FrameObj fr)
+{ if ( isProtectedObj(fr) )	/* a special (reusable) frame */
+  { if ( destroyCompleterFrame(fr) )
+      return;
+  }
+  send(fr, NAME_destroy, EAV);
+}
+
 
 static status
 unlinkFrame(FrameObj fr)
@@ -111,7 +121,7 @@ unlinkFrame(FrameObj fr)
 
     ws_enable_modal(fr, ON);
     if ( notNil(fr->transients) )
-      for_chain(fr->transients, sfr, send(sfr, NAME_destroy, EAV));
+      for_chain(fr->transients, sfr, destroyTransientFrame(sfr));
     if ( notNil(fr->transient_for) && notNil(fr->transient_for->transients) )
       send(fr->transient_for, NAME_detachTransient, fr, EAV);
 
@@ -985,7 +995,8 @@ static status
 labelFrame(FrameObj fr, Name label, Name icon)
 { assign(fr, label, label);
 
-  ws_set_label_frame(fr);
+  if ( ws_created_frame(fr) )
+    sdl_send(fr, NAME_SdlSetLabel, false, EAV);
 
   if ( notDefault(icon) )
     iconLabelFrame(fr, icon);
@@ -994,6 +1005,13 @@ labelFrame(FrameObj fr, Name label, Name icon)
 }
 
 
+static status
+SdlSetLabelFrame(FrameObj fr)
+{ ws_set_label_frame(fr);
+  succeed;
+}
+
+
 static status
 appendFrame(FrameObj fr, PceWindow sw)
 { return frameWindow(sw, fr);
@@ -1205,13 +1223,13 @@ inputWindowFrame(FrameObj fr, PceWindow iw)
 { PceWindow ow;
 
   if ( (ow=getHyperedObject(fr, NAME_inputWindow, DEFAULT)) && ow != iw )
-  { inputFocusWindow(ow, OFF);
+  { send(ow, NAME_inputFocus, OFF, EAV);
     freeHypersObject(fr, NAME_inputWindow, DEFAULT);
   }
 
   if ( fr->input_focus == ON && notNil(iw) )
   { newObject(ClassHyper, fr, iw, NAME_inputWindow, EAV);
-    inputFocusWindow(iw, ON);
+    send(iw, NAME_inputFocus, ON, EAV);
   }
 
   succeed;
@@ -1232,12 +1250,10 @@ inputFocusFrame(FrameObj fr, BoolObj val)
     { if ( (iw = getKeyboardFocusFrame(fr)) ||
 	   (iw = ws_window_holding_point_frame(fr)) )
 	inputWindowFrame(fr, iw);
-    } else if ( (iw = getKeyboardFocusFrame(fr)) )
-    { inputFocusWindow(iw, OFF);
     } else
     { Cell cell;
       for_cell(cell, fr->members)
-	inputFocusWindow(cell->value, OFF);
+	send(cell->value, NAME_inputFocus, OFF, EAV);
     }
   }
 
@@ -1790,6 +1806,8 @@ static senddecl send_frame[] =
      NAME_icon, "Set image and icon_label"),
   SM(NAME_label, 2, T_label, labelFrame,
      NAME_label, "Set label of the frame"),
+  SM(NAME_SdlSetLabel, 0, NULL, SdlSetLabelFrame,
+     NAME_thread, "Update the label of an open frame"),
   SM(NAME_fit, 0, NULL, fitFrame,
      NAME_layout, "Recompute windows and resize the frame"),
   SM(NAME_resize, 0, NULL, resizeFrame,
