@@ -478,7 +478,7 @@ distribute_stretches(stretch *s, int n, int w)
 	  grow_this = 0;
 	else
 	  grow_this = (total_shrink ==0 ? grow / is_pos
-				        : (grow * s[i].shrink) / total_shrink);
+					: (grow * s[i].shrink) / total_shrink);
       }
       s[i].size = s[i].ideal + grow_this;
       growed += grow_this;
@@ -828,8 +828,6 @@ layoutTile(TileObj t, Int ax, Int ay, Int aw, Int ah)
     h -= border*2;
   }
 
-  updateAdjusterPositionTile(t);
-
   if ( t->orientation == NAME_none )
     return send(t->object, NAME_doSet,
 		toInt(x), toInt(y), toInt(w), toInt(h), EAV);
@@ -1031,45 +1029,51 @@ out:
   answer(t->canResize);
 }
 
+void *
+forResizeAreaTile(TileObj t, for_tile_func func, Any ctx)
+{ if ( notNil(t->members) )
+  { Cell cell;
 
-static int
-adjust_pos(Int H, int bh)
-{ int h  = valInt(H);
-  int i1 = h*3/4;
-  int i2 = h - 30;
-  int i  = max(i1, i2);
+    for_cell(cell, t->members)
+    { TileObj t2 = cell->value;
+      TileObj t3;
+      void *rc = forResizeAreaTile(t2, func, ctx);
 
-  return i;
-}
+      if ( rc )
+	return rc;
 
-status
-updateAdjusterPositionTile(TileObj t)
-{ TileAdjuster adj;
+      if ( notNil(cell->next) )
+	t3 = cell->next->value;
+      else
+	break;
 
-  if ( notNil((adj = t->adjuster)) )
-  { int cx, cy, bw, bh;
-    int border;
+      if ( t->orientation == NAME_horizontal )
+      { if ( getCanResizeTile(t2) == ON )
+	{ int x0 = valInt(t2->area->x) + valInt(t2->area->w);
+	  int x1 = valInt(t3->area->x);
+	  void *rc = (*func)(ctx, t2,
+			     toInt(x0), t->area->y,
+			     toInt(x1-x0), t->area->h);
 
-    if ( notNil(t->super) )
-      border = valInt(t->super->border);
-    else
-      border = 0;
+	  if ( rc )
+	    return rc;
+	}
+      } else
+      { if ( getCanResizeTile(t2) == ON )
+	{ int y0 = valInt(t2->area->y) + valInt(t2->area->h);
+	  int y1 = valInt(t3->area->y);
+	  void *rc = (*func)(ctx, t2,
+			     t->area->x, toInt(y0),
+			     t->area->w, toInt(y1-y0));
 
-    bw = valInt(adj->area->w);
-    bh = valInt(adj->area->h);
-
-    if ( adj->orientation == NAME_horizontal )
-    { cx = valInt(t->area->x) + valInt(t->area->w) + border/2;
-      cy = valInt(t->area->y) + adjust_pos(t->area->h, bh);
-    } else
-    { cy = valInt(t->area->y) + valInt(t->area->h) + border/2;
-      cx = valInt(t->area->x) + adjust_pos(t->area->w, bw);
+	  if ( rc  )
+	    return rc;
+	}
+      }
     }
-
-    send(adj, NAME_geometry, toInt(cx-bw/2), toInt(cy-bh/2), EAV);
   }
 
-  succeed;
+  return NULL;
 }
 
 
@@ -1161,8 +1165,6 @@ static vardecl var_tile[] =
      NAME_resize, "Encouragement to get lower"),
   IV(NAME_canResize, "[bool]", IV_SEND,
      NAME_resize, "Can be resized by user?"),
-  IV(NAME_adjuster, "tile_adjuster*", IV_GET,
-     NAME_resize, "Window to adjust my size"),
   IV(NAME_border, "int", IV_BOTH,
      NAME_appearance, "Distance between areas"),
   IV(NAME_orientation, "{none,horizontal,vertical}", IV_GET,
@@ -1219,9 +1221,7 @@ static senddecl send_tile[] =
   SM(NAME_right, 2, T_associate, rightTile,
      NAME_layout, "Place a tile to my right"),
   SM(NAME_compute, 0, NULL, computeTile,
-     NAME_update, "Compute ideal sizes from sub-tiles"),
-  SM(NAME_updateAdjusterPosition, 0, NULL, updateAdjusterPositionTile,
-     NAME_resize, "Update ->geometry of <-adjuster")
+     NAME_update, "Compute ideal sizes from sub-tiles")
 };
 
 /* Get Methods */
@@ -1238,7 +1238,7 @@ static getdecl get_tile[] =
 /* Resources */
 
 static classvardecl rc_tile[] =
-{ RC(NAME_border, "int", "3",
+{ RC(NAME_border, "int", "4",
      "Border between subtiles")
 };
 
@@ -1257,4 +1257,3 @@ status
 makeClassTile(Class class)
 { return declareClass(class, &tile_decls);
 }
-
