@@ -114,7 +114,11 @@ gmp_too_big(DECL_LD gmp_tb why)
       max.value.i = LD->gmp.max_integer_size;
       PL_error(NULL, 0, "requires more than max_integer_size bytes",
 	       ERR_AR_TRIPWIRE, ATOM_max_integer_size, &max);
+#if O_THROW
       PL_rethrow();
+#else
+      PL_fatal_error("Big number retraint not supported without PL_throw()");
+#endif
     }
     case GMP_TB_STACK:
       outOfStack((Stack)&LD->stacks.global, STACK_OVERFLOW_THROW);
@@ -122,12 +126,20 @@ gmp_too_big(DECL_LD gmp_tb why)
     case GMP_TB_MALLOC:
     default:
       PL_no_memory();
+#if O_THROW
       PL_rethrow();
+#else
+      PL_fatal_error("Too large big number");
+#endif
   }
 
   abortProlog();		/* Just in case the above fails */
+#if O_THROW
   PL_rethrow();
-  return FALSE;
+#else
+  PL_fatal_error("Too large big number");
+#endif
+  return false;
 }
 
 
@@ -136,16 +148,16 @@ gmp_check_size(DECL_LD size_t bytes)
 { gmp_tb why = GMO_TB_OK;
 
   if ( bytes <= 1000 )
-    return TRUE;
+    return true;
   if ( bytes > LD->gmp.max_integer_size )
     why = GMP_TB_RESTRAINT;
   else if ( bytes > (size_t)globalStackLimit())
     why = GMP_TB_STACK;
   else
-    return TRUE;
+    return true;
 
   gmp_too_big(why);
-  return FALSE;
+  return false;
 }
 
 
@@ -364,7 +376,7 @@ mp_test_alloc(void)
 
   AR_END();
 
-  return TRUE;
+  return true;
 }
 #endif
 
@@ -481,7 +493,7 @@ globalMPZ(DECL_LD Word at, mpz_t mpz, int flags)
     if ( !hasGlobalSpace(wsz+MPZ_STACK_EXTRA+2) )
     { int rc = ensureGlobalSpace(wsz+MPZ_STACK_EXTRA+2, flags);
 
-      if ( rc != TRUE )
+      if ( rc != true )
 	return rc;
     }
     p = gTop;
@@ -509,7 +521,7 @@ globalMPZ(DECL_LD Word at, mpz_t mpz, int flags)
     *at = consPtr(p, TAG_INTEGER|STG_GLOBAL);
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -539,7 +551,7 @@ globalMPQ(DECL_LD Word at, mpq_t mpq, int flags)
     if ( !hasGlobalSpace(num_wsz+den_wsz+2+2*MPZ_STACK_EXTRA) )
     { int rc = ensureGlobalSpace(num_wsz+den_wsz+2+2*MPZ_STACK_EXTRA, flags);
 
-      if ( rc != TRUE )
+      if ( rc != true )
 	return rc;
     }
     p = gTop;
@@ -578,7 +590,7 @@ globalMPQ(DECL_LD Word at, mpq_t mpq, int flags)
     *at = consPtr(p, TAG_INTEGER|STG_GLOBAL);
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -698,18 +710,18 @@ get_mpq_from_code(Code pc, mpq_t mpq)
 }
 
 
-int
+bool
 get_int64(DECL_LD word w, int64_t *ip)
 { if ( tagex(w) == (TAG_INTEGER|STG_INLINE) )
   { *ip = valInt(w);
-    return TRUE;
+    return true;
   } else if ( tagex(w) == (TAG_INTEGER|STG_GLOBAL) )
   { number n;
 
     get_bigint(w, &n);
     return mpz_to_int64(n.value.mpz, ip);
   } else
-    return FALSE;
+    return false;
 }
 
 
@@ -816,9 +828,9 @@ load_abs_mpz_size(const char *data, int *szp, int *neg)
   if ( neg )
   { if ( size < 0 )
     { size = -size;
-      *neg = TRUE;
+      *neg = true;
     } else
-    { *neg = FALSE;
+    { *neg = false;
     }
   } else if ( size < 0 )
     size = -size;
@@ -1056,7 +1068,7 @@ promoteToMPZNumber(number *n)
       break;
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -1091,7 +1103,7 @@ promoteToMPQNumber(number *n)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -1170,7 +1182,7 @@ clearGMPNumber(Number n)
 void
 initGMP(void)
 { if ( !GD->gmp.initialised )
-  { GD->gmp.initialised = TRUE;
+  { GD->gmp.initialised = true;
 
 #if O_BF
     initBF();
@@ -1209,7 +1221,7 @@ initGMP(void)
 void
 cleanupGMP(void)
 { if ( GD->gmp.initialised )
-  { GD->gmp.initialised = FALSE;
+  { GD->gmp.initialised = false;
 
 #ifdef O_MY_GMP_ALLOC
     if ( !GD->gmp.keep_alloc_functions )
@@ -1304,10 +1316,10 @@ mpz_to_int64(mpz_t mpz, int64_t *i)
     else
       *i = (int64_t)v;
 
-    return TRUE;
+    return true;
   }
 
-  return FALSE;
+  return false;
 }
 
 /* return: <0:              -1
@@ -1388,13 +1400,13 @@ put_mpz(DECL_LD Word at, mpz_t mpz, int flags)
     if ( !hasGlobalSpace(0) )		/* ensure we have room for bindConst */
     { int rc = ensureGlobalSpace(0, flags);
 
-      if ( rc != TRUE )
+      if ( rc != true )
 	return rc;
     }
 
     *at = consInt(v);
     assert(valInt(*at) == v);
-    return TRUE;
+    return true;
   } else
   { return globalMPZ(at, mpz, flags);
   }
@@ -1404,8 +1416,8 @@ put_mpz(DECL_LD Word at, mpz_t mpz, int flags)
 
 /* returns one of
 
-  TRUE: ok
-  FALSE: some error
+  true: ok
+  false: some error
   GLOBAL_OVERFLOW: no space
   LOCAL_OVERFLOW: cannot represent (no GMP)
 */
@@ -1417,7 +1429,7 @@ put_int64(DECL_LD Word at, int64_t l, int flags)
   r = consInt(l);
   if ( valInt(r) == l )
   { *at = r;
-    return TRUE;
+    return true;
   } else
   {
 #ifdef O_BIGNUM
@@ -1458,13 +1470,13 @@ affected by GC/shift.  The intented scenario is:
 
   { word c;
 
-    if ( (rc=put_number(&c, n, ALLOW_GC)) == TRUE )
+    if ( (rc=put_number(&c, n, ALLOW_GC)) == true )
       bindConst(<somewhere>, c);
     ...
   }
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
+int				/* true, false, _*OVERFLOW */
 put_number(DECL_LD Word at, Number n, int flags)
 { switch(n->type)
   { case V_INTEGER:
@@ -1474,12 +1486,12 @@ put_number(DECL_LD Word at, Number n, int flags)
       { if ( !hasGlobalSpace(0) )
 	{ int rc = ensureGlobalSpace(0, flags);
 
-	  if ( rc != TRUE )
+	  if ( rc != true )
 	    return rc;
 	}
 
 	*at = w;
-	return TRUE;
+	return true;
       }
 
       return put_int64(at, n->value.i, flags);
@@ -1499,7 +1511,7 @@ put_number(DECL_LD Word at, Number n, int flags)
   }
 
   assert(0);
-  return FALSE;
+  return false;
 }
 
 
@@ -1518,14 +1530,14 @@ PL_unify_number(DECL_LD term_t t, Number n)
   if ( canBind(*p) )
   { int rc;
 
-    if ( (rc=put_number(&w, n, ALLOW_GC)) != TRUE )
+    if ( (rc=put_number(&w, n, ALLOW_GC)) != true )
       return raiseStackOverflow(rc);
 
     p = valTermRef(t);			/* put_number can shift the stacks */
     deRef(p);
 
     bindConst(p, w);
-    return TRUE;
+    return true;
   }
 
   switch(n->type)
@@ -1580,12 +1592,12 @@ PL_put_number(DECL_LD term_t t, Number n)
 { word w;
   int rc;
 
-  if ( (rc=put_number(&w, n, ALLOW_GC)) != TRUE )
+  if ( (rc=put_number(&w, n, ALLOW_GC)) != true )
     return raiseStackOverflow(rc);
 
   *valTermRef(t) = w;
 
-  return TRUE;
+  return true;
 }
 
 
@@ -1627,7 +1639,7 @@ API_STUB(int)
 		 *	     PROMOTION		*
 		 *******************************/
 
-int
+bool
 promoteToFloatNumber(Number n)
 { switch(n->type)
   { case V_INTEGER:
@@ -1653,18 +1665,18 @@ promoteToFloatNumber(Number n)
     }
 #endif
     case V_FLOAT:
-      break;
+      return true;
   }
 
   return check_float(n);
 }
 
 
-int
+bool
 promoteNumber(Number n, numtype t)
 { switch(t)
   { case V_INTEGER:
-      return TRUE;
+      return true;
 #ifdef O_BIGNUM
     case V_MPZ:
       return promoteToMPZNumber(n);
@@ -1675,19 +1687,19 @@ promoteNumber(Number n, numtype t)
       return promoteToFloatNumber(n);
     default:
       assert(0);
-      return FALSE;
+      return false;
   }
 }
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-same_type_numbers(n1, n2)
+make_same_type_numbers(n1, n2)
     Upgrade both numbers to the `highest' type of both. Number types are
     defined in the enum-type numtype, which is supposed to define a
     total ordering between the number types.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
+bool
 make_same_type_numbers(Number n1, Number n2)
 { if ( (int)n1->type > (int)n2->type )
     return promoteNumber(n2, n1->type);
@@ -2242,7 +2254,7 @@ mpq_set_double(mpq_t r, double f)	/* float -> nice rational */
 		 *	 PUBLIC INTERFACE	*
 		 *******************************/
 
-int
+bool
 PL_get_mpz(term_t t, mpz_t mpz)
 { GET_LD
   Word p = valTermRef(t);
@@ -2265,14 +2277,14 @@ PL_get_mpz(term_t t, mpz_t mpz)
 	assert(0);
     }
 
-    return TRUE;
+    return true;
   }
 
-  return FALSE;
+  return false;
 }
 
 
-int
+bool
 PL_get_mpq(term_t t, mpq_t mpq)
 { if ( PL_is_rational(t) )
   { GET_LD
@@ -2285,28 +2297,28 @@ PL_get_mpq(term_t t, mpq_t mpq)
     { case V_INTEGER:
 	if ( n.value.i >= LONG_MIN && n.value.i <= LONG_MAX )
 	{ mpq_set_si(mpq, (long)n.value.i, 1L);
-	  return TRUE;
+	  return true;
 	}
 	promoteToMPZNumber(&n);
 	/*FALLTHROUGH*/
       case V_MPZ:
 	mpq_set_z(mpq, n.value.mpz);
 	clearNumber(&n);
-	return TRUE;
+	return true;
       case V_MPQ:
 	mpq_set(mpq, n.value.mpq);
 	clearNumber(&n);
-	return TRUE;
+	return true;
       default:
 	;
     }
   }
 
-  return FALSE;
+  return false;
 }
 
 
-int
+bool
 PL_unify_mpz(term_t t, mpz_t mpz)
 { GET_LD
   number n;
@@ -2323,7 +2335,7 @@ PL_unify_mpz(term_t t, mpz_t mpz)
 }
 
 
-int
+bool
 PL_unify_mpq(term_t t, mpq_t mpq)
 { GET_LD
   number n;
